﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CentralMudances.Servei;
using System.IO;

namespace CentralMudances.Controls
{
    /// <summary>
    /// Lógica de interacción para MantUsuaris.xaml
    /// </summary>
    public partial class DetallPressupost : UserControl
    {
        Servei.ServeiMudancesClient client = new Servei.ServeiMudancesClient();

        private Usuari _usuariActual = null;
        private Pressupost _pressupostActual = null;
        private Operari _operariSeleccionat = null;
        private Vehicle _vehicleSeleccionat = null;
        private LiniaPressupost _liniaSeleccionada = null;
        private Imatge _imatgeSeleccionada = null;

        /// <summary>
        /// Constructor de la classe
        /// </summary>
        public DetallPressupost(Usuari usuariActual, Pressupost pressupostActual)
        {
            InitializeComponent();

            this._usuariActual = usuariActual;
            this._pressupostActual = pressupostActual;

            if (_pressupostActual.Id == 0)
            {
                lPressupost.Content = "Nou Pressupost";
            }
            else
            {
                lPressupost.Content = "Pressupost " + _pressupostActual.Id.ToString();
            }
        }

        /// <summary>
        /// Funció que salta qual el control s'ha carregat.
        /// </summary>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            if (_pressupostActual.Id != 0)
                this.LoadPressupost();

            this.MostraPressupost();
        }

        /// <summary>
        /// Funció encarregada de la càrrega dels usuaris donats d'alta al sistema.
        /// </summary>
        private void LoadPressupost()
        {
            _pressupostActual = client.getPressupostById(_pressupostActual.Id);
        }

        /// <summary>
        /// Funció encarregada de mostrar la pantalla de cerca per tal de buscar un usuari i poder-lo assignar.
        /// </summary>
        private void bUsuari_Click(object sender, RoutedEventArgs e)
        {
            Dictionary<int, string> valors = new Dictionary<int, string>();

            List<Usuari> usuaris = client.getUsers().ToList();
            var result = from i in usuaris
                         where i.baixa == false
                         && (i.wp8 == true
                         || i.administrador == true)
                         select i;

            foreach (Usuari usr in result)
            {
                valors.Add(usr.Id, usr.nom);
            }

            Finestres.CercaElements cerca = new Finestres.CercaElements("Usuaris", valors);
            cerca.ShowDialog();
            if (cerca.selectedValue == -1)
            {
                MessageBox.Show("No s'ha sel·leccionat cap Usuari");
            }
            else
            {
                _pressupostActual.usuari = (from i in usuaris
                                            where i.Id == cerca.selectedValue
                                            select i).FirstOrDefault();
                this.MostraPressupost();
            }
        }
        /// <summary>
        /// Funció encarregada de mostrar la pantalla de cerca per tal de buscar un client i poder-lo assignar.
        /// </summary>
        private void bClient_Click(object sender, RoutedEventArgs e)
        {
            Dictionary<int, string> valors = new Dictionary<int, string>();

            List<Client> clients = client.getClients().ToList();
            var result = from i in clients
                         where i.baixa == false
                         select i;

            foreach (Client cli in result)
            {
                valors.Add(cli.Id, cli.nom + " " + cli.cognoms);
            }

            Finestres.CercaElements cerca = new Finestres.CercaElements("Clients", valors);
            cerca.ShowDialog();
            if (cerca.selectedValue == -1)
            {
                MessageBox.Show("No s'ha sel·leccionat cap Client");
            }
            else
            {
                _pressupostActual.client = (from i in clients
                                            where i.Id == cerca.selectedValue
                                            select i).FirstOrDefault();

                _pressupostActual.adreça_origen = _pressupostActual.client.adreça;
                _pressupostActual.cp_origen = _pressupostActual.client.cp;
                _pressupostActual.poblacio_origen = _pressupostActual.client.poblacio;
                _pressupostActual.provincia_origen = _pressupostActual.client.provincia;

                this.MostraPressupost();
            }
        }

        #region "Control Operaris"
        private void bAddOperari_Click(object sender, RoutedEventArgs e)
        {
            this.GuardaCanvis(false);

            Dictionary<int, string> valors = new Dictionary<int, string>();
            List<int> elemensAssignats = new List<int>();

            if (_pressupostActual.operaris != null &&
                _pressupostActual.operaris.Count() > 0)
            {
                elemensAssignats = (from i in _pressupostActual.operaris
                                    select i.Id).ToList();
            }

            List<Operari> operaris = client.getOperaris().ToList();
            var result = from i in operaris
                         where i.baixa == false
                         && !elemensAssignats.Contains(i.Id)
                         select i;

            foreach (Operari op in result)
            {
                valors.Add(op.Id, op.nom);
            }

            Finestres.CercaElements cerca = new Finestres.CercaElements("Operaris", valors);
            cerca.ShowDialog();
            if (cerca.selectedValue == -1)
            {
                MessageBox.Show("No s'ha sel·leccionat cap Operari");
            }
            else
            {
                List<Operari> temp;

                if (_pressupostActual.operaris == null)
                {
                    temp = new List<Operari>();
                }
                else
                {
                    temp = _pressupostActual.operaris.ToList();
                }

                temp.Add((from i in operaris where i.Id == cerca.selectedValue select i).First());
                _pressupostActual.operaris = temp.ToArray();

                this.MostraPressupost();
            }
        }

        private void bRemoveOperari_Click(object sender, RoutedEventArgs e)
        {
            this.GuardaCanvis(false);

            if (_operariSeleccionat != null)
            {
                if (MessageBox.Show("Vol esborrar l'operari seleccionat?", "Confirmació", MessageBoxButton.YesNo) == MessageBoxResult.Yes)
                {
                    List<Operari> temp = _pressupostActual.operaris.ToList();
                    temp.Remove(_operariSeleccionat);
                    _pressupostActual.operaris = temp.ToArray();
                    this.MostraPressupost();
                }
            }
        }

        private void GOperaris_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
            {
                _operariSeleccionat = null;
                return;
            }

            _operariSeleccionat = (Operari)e.AddedItems[0];
        }
        #endregion

        #region "Control Vehicles"
        private void bAddVehicle_Click(object sender, RoutedEventArgs e)
        {
            this.GuardaCanvis(false);

            Dictionary<int, string> valors = new Dictionary<int, string>();
            List<int> elemensAssignats = new List<int>();

            if (_pressupostActual.vehicles != null &&
                _pressupostActual.vehicles.Count() > 0)
            {
                elemensAssignats = (from i in _pressupostActual.vehicles
                                    select i.Id).ToList();
            }

            List<Vehicle> vehicles = client.getVehicles().ToList();
            var result = from i in vehicles
                         where i.baixa == false
                         && !elemensAssignats.Contains(i.Id)
                         select i;

            foreach (Vehicle veh in result)
            {
                valors.Add(veh.Id, veh.nom);
            }

            Finestres.CercaElements cerca = new Finestres.CercaElements("Vehicles", valors);
            cerca.ShowDialog();
            if (cerca.selectedValue == -1)
            {
                MessageBox.Show("No s'ha sel·leccionat cap Vehicle");
            }
            else
            {
                List<Vehicle> temp;

                if (_pressupostActual.vehicles == null)
                {
                    temp = new List<Vehicle>();
                }
                else
                {
                    temp = _pressupostActual.vehicles.ToList();
                }

                temp.Add((from i in vehicles where i.Id == cerca.selectedValue select i).First());
                _pressupostActual.vehicles = temp.ToArray();

                this.MostraPressupost();
            }
        }

        private void bRemoveVehicle_Click(object sender, RoutedEventArgs e)
        {
            this.GuardaCanvis(false);

            if (_vehicleSeleccionat != null)
            {
                if (MessageBox.Show("Vol esborrar el vehicle seleccionat?", "Confirmació", MessageBoxButton.YesNo) == MessageBoxResult.Yes)
                {
                    List<Vehicle> temp = _pressupostActual.vehicles.ToList();
                    temp.Remove(_vehicleSeleccionat);
                    _pressupostActual.vehicles = temp.ToArray();
                    this.MostraPressupost();
                }
            }
        }

        private void GVehicles_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
            {
                _vehicleSeleccionat = null;
                return;
            }

            _vehicleSeleccionat = (Vehicle)e.AddedItems[0];
        }
        #endregion

        #region "Control Linies"
        private bool ValidaDadesLinia()
        {
            int tempsDedicat;
            decimal preu;

            if (!int.TryParse(tTempsDedicat.Text, out tempsDedicat))
            {
                MessageBox.Show("El temps dedicat introduït no es numèric!", "Error", MessageBoxButton.OK);
                return false;
            }
            if (!decimal.TryParse(tPreu.Text, out preu))
            {
                MessageBox.Show("El preu introduït no es correcte!", "Error", MessageBoxButton.OK);
                return false;
            }
            return true;

        }
        private void bAddLinia_Click(object sender, RoutedEventArgs e)
        {
            _liniaSeleccionada = new LiniaPressupost();
            _liniaSeleccionada.Id = -1;
            this.NetejaLinies();
        }
        private void bGuardaCanvis_Click(object sender, RoutedEventArgs e)
        {
            if (!ValidaDadesLinia())
                return;

            _liniaSeleccionada.article = tArticle.Text;
            _liniaSeleccionada.preu = decimal.Parse(tPreu.Text);
            _liniaSeleccionada.comentaris = tComentaris.Text;
            _liniaSeleccionada.temps_dedicat = int.Parse(tTempsDedicat.Text);
            _liniaSeleccionada.transportat = ckTransportat.IsChecked.Value;

            if (_liniaSeleccionada.Id == -1)
            {
                List<LiniaPressupost> temp;

                if (_pressupostActual.linies == null)
                {
                    temp = new List<LiniaPressupost>();
                }
                else
                {
                    temp = _pressupostActual.linies.ToList();
                }

                _liniaSeleccionada.Id = 0;
                temp.Add(_liniaSeleccionada);
                _pressupostActual.linies = temp.ToArray();
            }

            this.MostraPressupost();
        }
        private void GLinies_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;

            if (_liniaSeleccionada != null &&
                _liniaSeleccionada.Id == -1)
            {
                if (MessageBox.Show("Està creant una línia, segur que vol descartar els canvis?", "Confirmació", MessageBoxButton.YesNo) == MessageBoxResult.No)
                {
                    return;
                }
            }

            _liniaSeleccionada = (LiniaPressupost)e.AddedItems[0];
            this.PintaLinia();
        }
        private void bRemoveLinia_Click(object sender, RoutedEventArgs e)
        {
            if (_liniaSeleccionada != null)
            {
                if (MessageBox.Show("Segur que vol esborrar la linia seleccionada?", "Confirmació", MessageBoxButton.YesNo) == MessageBoxResult.Yes)
                {
                    List<LiniaPressupost> temp = _pressupostActual.linies.ToList();
                    temp.Remove(_liniaSeleccionada);
                    _pressupostActual.linies = temp.ToArray();
                    this.MostraPressupost();
                }
            }
        }
        #endregion

        /// <summary>
        /// Funció encarregada d'esborrar les dades de les línies que es mostren en pantalla.
        /// </summary>
        private void NetejaLinies()
        {
            tArticle.Text = "";
            tPreu.Text = "0";
            tComentaris.Text = "";
            tTempsDedicat.Text = "0";
            ckTransportat.IsChecked = false;
            GImatges.ItemsSource = null;
        }
        /// <summary>
        /// Funció encarregada de mostrar per pantalla les dades de la línia que s'ha seleccionat.
        /// </summary>
        private void PintaLinia()
        {
            if (_liniaSeleccionada == null)
                return;

            tArticle.Text = _liniaSeleccionada.article;
            tComentaris.Text = _liniaSeleccionada.comentaris;
            tPreu.Text = _liniaSeleccionada.preu.ToString();
            tTempsDedicat.Text = _liniaSeleccionada.temps_dedicat.ToString();
            ckTransportat.IsChecked = _liniaSeleccionada.transportat;
            GImatges.ItemsSource = _liniaSeleccionada.imatges;
        }
        /// <summary>
        /// Funció encarregada de mostrar en pantalla les dades del pressupost seleccionat.
        /// </summary>
        private void MostraPressupost()
        {
            this.NetejaLinies();

            tId.Text = _pressupostActual.Id.ToString();
            ckAcceptat.IsChecked = _pressupostActual.acceptat;
            tData.SelectedDate = _pressupostActual.data;

            if(_pressupostActual.usuari != null) 
                tUsuari.Text = _pressupostActual.usuari.nom;
            
            if (_pressupostActual.client != null) 
                tClient.Text = _pressupostActual.client.nom + " " + _pressupostActual.client.cognoms;
            
            tDireccioOrigen.Text = _pressupostActual.adreça_origen;
            tCPOrigen.Text = _pressupostActual.cp_origen;
            tPoblacioOrigen.Text = _pressupostActual.poblacio_origen;
            tProvinciaOrigen.Text = _pressupostActual.provincia_origen;

            tDireccioDesti.Text = _pressupostActual.adreça_desti;
            tCPDesti.Text = _pressupostActual.cp_desti;
            tPoblacioDesti.Text = _pressupostActual.poblacio_desti;
            tProvinciaDesti.Text = _pressupostActual.provincia_desti;

            GOperaris.ItemsSource = _pressupostActual.operaris;
            GVehicles.ItemsSource = _pressupostActual.vehicles;
            GLinies.ItemsSource = _pressupostActual.linies;

            this.AssignaPermisos();

            ckAcceptat.Checked += ckAcceptat_Checked;
        }

        /// <summary>
        /// Funció encarregada d'habilitar o deshabilitar els elements del formulari en funció de l'estat del pressupost.
        /// </summary>
        private void AssignaPermisos()
        {
            if (ckAcceptat.IsChecked.Value || _pressupostActual.facturat)
            {
                tId.IsEnabled = false;
                ckAcceptat.IsEnabled = false;
                tData.IsEnabled = false;

                tUsuari.IsEnabled = false;
                bUsuari.IsEnabled = false;

                tClient.IsEnabled = false;
                bClient.IsEnabled = false;

                tDireccioOrigen.IsEnabled = false;
                tCPOrigen.IsEnabled = false;
                tPoblacioOrigen.IsEnabled = false;
                tProvinciaOrigen.IsEnabled = false;

                tDireccioDesti.IsEnabled = false;
                tCPDesti.IsEnabled = false;
                tPoblacioDesti.IsEnabled = false;
                tProvinciaDesti.IsEnabled = false;

                GOperaris.IsEnabled = false;
                bAddOperari.IsEnabled = false;
                bRemoveOperari.IsEnabled = false;
                
                GVehicles.IsEnabled = false;
                bAddVehicle.IsEnabled = false;
                bRemoveVehicle.IsEnabled = false;

                bAddLinia.IsEnabled = false;
                bRemoveLinia.IsEnabled = false;
                tArticle.IsEnabled = false;
                tPreu.IsEnabled = false;
                tComentaris.IsEnabled = false;

                //bAddImage.IsEnabled = false;
                //bRemoveImage.IsEnabled = false;

                bFactura.Visibility = System.Windows.Visibility.Visible;
            }
            if (_pressupostActual.facturat)
            {
                bGuardaCanvis.IsEnabled = false;
                bSaveAll.IsEnabled = false;
                ckTransportat.IsEnabled = false;
                tTempsDedicat.IsEnabled = false;
                bFactura.Visibility = System.Windows.Visibility.Hidden;
            }
        }

        /// <summary>
        /// Funció encarregada de guardar tots els canvis fets al pressupost a la base de dades.
        /// </summary>
        private void bSaveAll_Click(object sender, RoutedEventArgs e)
        {
            if (!this.GuardaCanvis(true))
            {
                return;
            }

            if (_pressupostActual.Id == 0)
            {
                client.addPressupost(_pressupostActual);
            }
            else
            {
                client.updatePressupost(_pressupostActual);
            }
            MessageBox.Show("Modificacions guardades correctament", "Confirmació");
        }
        /// <summary>
        /// Funció encarregada de validar les dades introduïdes per l'usuari i assignar els canvis realitzats
        /// a l'objecte que posteriorment guardarem a la base de dades.
        /// </summary>
        /// <returns></returns>
        private bool GuardaCanvis(bool Valida)
        {
            if (_pressupostActual.client == null && Valida)
            {
                MessageBox.Show("No s'ha informat cap client. Es una dada obligatòria.", "Error", MessageBoxButton.OK);
                return false;
            }
            if (_pressupostActual.usuari == null && Valida)
            {
                MessageBox.Show("No s'ha informat cap usuari. Es una dada obligatòria.", "Error", MessageBoxButton.OK);
                return false;
            }

            _pressupostActual.acceptat = ckAcceptat.IsChecked.Value;
            _pressupostActual.data = tData.SelectedDate.Value;
            _pressupostActual.adreça_origen = tDireccioOrigen.Text;
            _pressupostActual.cp_origen = tCPOrigen.Text;
            _pressupostActual.poblacio_origen = tPoblacioOrigen.Text;
            _pressupostActual.provincia_origen = tProvinciaOrigen.Text;
            _pressupostActual.adreça_desti = tDireccioDesti.Text;
            _pressupostActual.cp_desti = tCPDesti.Text;
            _pressupostActual.poblacio_desti = tPoblacioDesti.Text;
            _pressupostActual.provincia_desti = tProvinciaDesti.Text;

            return true;
        }

        /// <summary>
        /// Funció que salta quan es selecciona una línia al grid.
        /// </summary>
        private void GImatges_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
            {
                iImatge.Source = null;
                return;
            }

            _imatgeSeleccionada = ((Imatge)e.AddedItems[0]);
            MemoryStream ms = new MemoryStream(_imatgeSeleccionada.imatge);
            
            BitmapImage bi = new BitmapImage();
            bi.BeginInit();
            bi.CreateOptions = BitmapCreateOptions.PreservePixelFormat;
            bi.CacheOption = BitmapCacheOption.OnLoad;
            bi.UriSource = null;
            bi.StreamSource = ms;
            bi.EndInit();
            bi.StreamSource = ms;
            iImatge.Source = bi;
        }

        /// <summary>
        /// Funció encarregada de mostrar per pantalla i imprimir l'informe corresponent a l'estat del pressupost.
        /// </summary>
        private void bPrint_Click(object sender, RoutedEventArgs e)
        {
            Finestres.ShowReport sr = new Finestres.ShowReport();
            sr.ID = _pressupostActual.Id;
            sr.Factura = _pressupostActual.facturat;
            sr.eMailDestí = _pressupostActual.client.email;
            sr.Show();
        }

        /// <summary>
        /// Funció que salta quan marquem el pressupost com acceptat.
        /// Demana confirmació, guarda les dades i el bloqueja per a que no es pugui modificar més.
        /// </summary>
        private void ckAcceptat_Checked(object sender, RoutedEventArgs e)
        {
            if (ckAcceptat.IsChecked.Value)
            {
                if (MessageBox.Show("Segur que vol acceptar el pressupost?\nNo es podrà tornar a modificar!", "Confirmació", MessageBoxButton.YesNo) == MessageBoxResult.Yes)
                {
                    this.GuardaCanvis(true);
                    this.MostraPressupost();
                }
                else
                {
                    MessageBox.Show("Acceptació cancel·lada per l'usuari.", "Confirmació", MessageBoxButton.OK);
                    ckAcceptat.IsChecked = false;
                }
            }
        }

        /// <summary>
        /// Funció encarregada de facturar el pressupost sempre que tots els articles hagin estat transportats.
        /// </summary>
        private void bFactura_Click(object sender, RoutedEventArgs e)
        {
            if (!this.EsFacturable())
            {
                if (MessageBox.Show("No s'han transportat totes les línies que s'havien de transportar!\nConfirma que vol facturar igualment?", "Confirmació", MessageBoxButton.YesNo) == MessageBoxResult.No)
                {
                    return;
                }
            }

            client.facturaPressupost(_pressupostActual.Id);
            _pressupostActual = client.getPressupostById(_pressupostActual.Id);
            this.MostraPressupost();

        }
        
        /// <summary>
        /// Funció encarregada de comprovar que el pressupost pot passar a factura sense problema.
        /// Si el número de línies sense transportar es 0, es podrà facturar.
        /// </summary>
        /// <returns></returns>
        private bool EsFacturable()
        {
            var numLins = (from i in _pressupostActual.linies
                           where i.transportat == false
                           select i).Count();

            return numLins == 0;
        }

    }
}
