﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CentralMudances.Servei;

namespace CentralMudances.Controls
{
    /// <summary>
    /// Lógica de interacción para MantUsuaris.xaml
    /// </summary>
    public partial class MantClients : UserControl
    {
        private List<Client> clients = new List<Client>();
        Servei.ServeiMudancesClient client = new Servei.ServeiMudancesClient();

        private Client _clientActual = null;

        /// <summary>
        /// Constructor de la classe
        /// </summary>
        public MantClients()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Funció encarregada de la càrrega dels clients donats d'alta al sistema.
        /// </summary>
        private void LoadClients()
        {
            clients = client.getClients().ToList();
            ListGrid.ItemsSource = clients;
        }

        /// <summary>
        /// Funció que salta qual el control s'ha carregat.
        /// </summary>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.LoadClients();
        }

        /// <summary>
        /// Funció que salta qual es selecciona una fila del grid de pantalla.
        /// S'encarrega de carregar l'objecte de treball actual.
        /// </summary>
        private void ListGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;

            _clientActual = (Client)e.AddedItems[0];
            this.MostraElementActual();

        }
        
        /// <summary>
        /// Funció encarregada de buidar tots els elements d'entrada de dades del formulari.
        /// </summary>
        private void BuidaFormulari()
        {
            t_nom.Text = "";
            t_cognoms.Text = "";
            t_dni.Text = "";
            t_poblacio.Text = "";
            t_provincia.Text = "";
            t_telefon.Text = "";
            t_mobil.Text = "";
            t_fax.Text = "";
            t_adreca.Text = "";
            t_cp.Text = "";
            t_email.Text = "";

            ck_baixa.IsChecked = false;
        }
        /// <summary>
        /// Funció encarregada d'emplenar els elements d'entrada de dades amb les dades de l'element actualment seleccionat.
        /// </summary>
        private void MostraElementActual()
        {
            this.BuidaFormulari();
            t_nom.Text = _clientActual.nom;
            t_cognoms.Text = _clientActual.cognoms;
            t_dni.Text = _clientActual.dni;
            t_poblacio.Text = _clientActual.poblacio;
            t_provincia.Text = _clientActual.provincia;
            t_telefon.Text = _clientActual.telefon;
            t_mobil.Text = _clientActual.mobil;
            t_fax.Text = _clientActual.fax;
            t_adreca.Text = _clientActual.adreça;
            t_cp.Text = _clientActual.cp;
            t_email.Text = _clientActual.email;
            ck_baixa.IsChecked = _clientActual.baixa;
        }
        /// <summary>
        /// Funció encarrega d'actualitzar l'objecte amb els que estem treballant amb le dades que l'usuari ha introduït.
        /// </summary>
        private void ActualitzaElementActual()
        {
            _clientActual.nom = t_nom.Text;
            _clientActual.cognoms = t_cognoms.Text;
            _clientActual.dni = t_dni.Text;
            _clientActual.poblacio = t_poblacio.Text;
            _clientActual.provincia = t_provincia.Text;
            _clientActual.telefon = t_telefon.Text;
            _clientActual.mobil = t_mobil.Text;
            _clientActual.fax = t_fax.Text;
            _clientActual.adreça = t_adreca.Text;
            _clientActual.cp = t_cp.Text;
            _clientActual.email = t_email.Text;
            _clientActual.baixa = ck_baixa.IsChecked.Value;
        }

        /// <summary>
        /// Funció encarregada de guardar les dades introduïdes per l'usuari.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void b_guarda_Click(object sender, RoutedEventArgs e)
        {
            if (_clientActual == null)
                return;

            this.ActualitzaElementActual();
            if (_clientActual.Id == 0)
            {
                client.addClient(_clientActual);
            }
            else
            {
                client.updateClient(_clientActual);
            }
            this.BuidaFormulari();
            this.LoadClients();
        }
        /// <summary>
        /// Funció encarregada de crear un nou element amb el que treballar i donar d'alta a la base de dades.
        /// </summary>
        private void b_nou_Click(object sender, RoutedEventArgs e)
        {
            this.BuidaFormulari();
            _clientActual = new Client();
        }

    }
}
