﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CentralMudances.Servei;

namespace CentralMudances.Controls
{
    /// <summary>
    /// Lógica de interacción para MantUsuaris.xaml
    /// </summary>
    public partial class MantConfiguracio : UserControl
    {
        private List<Configuracio> configs = new List<Configuracio>();
        Servei.ServeiMudancesClient client = new Servei.ServeiMudancesClient();

        private Operari _operariActual = null;

        /// <summary>
        /// Constructor de la classe
        /// </summary>
        public MantConfiguracio()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Funció encarregada de la càrrega de la configuració donada d'alta al sistema.
        /// </summary>
        private void LoadConfiguracio()
        {
            configs = client.getConfiguracio().ToList();
        }

        /// <summary>
        /// Funció que salta qual el control s'ha carregat.
        /// </summary>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.LoadConfiguracio();
            this.MostraConfiguracio();
        }

        /// <summary>
        /// Funció encarregada de mostrar per pantalla la configuració retornada.
        /// </summary>
        private void MostraConfiguracio()
        {
            foreach (Configuracio c in configs)
            {
                switch (c.propietat)
                { 
                    case "mailServer":
                        tCorreu.Text = c.valor;
                        break;
                    case "eMail":
                        tEmail.Text = c.valor;
                        break;
                    case "emailUser":
                        tUsuari.Text = c.valor;
                        break;
                    case "emailPassword":
                        tPassword.Text = c.valor;
                        break;
                    case "reportServer":
                        tReportServer.Text = c.valor;
                        break;
                    case "reportPressupost":
                        tReportPressupost.Text = c.valor;
                        break;
                    case "reportFactura":
                        tReportFactura.Text = c.valor;
                        break;
                    case "reportUser":
                        tReportUser.Text = c.valor;
                        break;
                    case "reportPassword":
                        tReportPassword.Text = c.valor;
                        break;
                }
            }
        }

        /// <summary>
        /// Funció encarregada d'actualitzar o crear un nou registre de configuració de l'aplicació.
        /// </summary>
        private void ActualitzaOCreaConfiguracio(string propietat, string valor)
        {
            Configuracio cnf;

            cnf = (from i in configs where i.propietat == propietat select i).FirstOrDefault();
            if (cnf != null)
            {
                cnf.valor = valor;
            }
            else
            {
                Configuracio newConfig = new Configuracio();
                newConfig.propietat = propietat;
                newConfig.valor = valor;
                configs.Add(newConfig);
            }
        }

        /// <summary>
        /// Funció encarregada de guardar la configuració de l'aplicació.
        /// </summary>
        private void GuardaConfiguracio()
        {
            this.ActualitzaOCreaConfiguracio("mailServer", tCorreu.Text);
            this.ActualitzaOCreaConfiguracio("eMail", tEmail.Text);
            this.ActualitzaOCreaConfiguracio("emailUser", tUsuari.Text);
            this.ActualitzaOCreaConfiguracio("emailPassword", tPassword.Text);
            this.ActualitzaOCreaConfiguracio("reportServer", tReportServer.Text);
            this.ActualitzaOCreaConfiguracio("reportPressupost", tReportPressupost.Text);
            this.ActualitzaOCreaConfiguracio("reportFactura", tReportFactura.Text);
            this.ActualitzaOCreaConfiguracio("reportUser", tReportUser.Text);
            this.ActualitzaOCreaConfiguracio("reportPassword", tReportPassword.Text);
            client.updateConfiguracio(configs.ToArray());
        }

        /// <summary>
        /// Funció encarregada de la crida a la funció GuardaConfiguració.
        /// </summary>
        private void b_guarda_Click(object sender, RoutedEventArgs e)
        {
            this.GuardaConfiguracio();
            MessageBox.Show("Dades guardades correctament", "Confirmació");
        }
    }
}
