﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CentralMudances.Servei;

namespace CentralMudances.Controls
{
    /// <summary>
    /// Lógica de interacción para MantUsuaris.xaml
    /// </summary>
    public partial class MantPressupostos : UserControl
    {
        private Usuari usuariActual;

        private List<Pressupost> pressupostos = new List<Pressupost>();
        Servei.ServeiMudancesClient client = new Servei.ServeiMudancesClient();
        private Pressupost _pressupostActual = null;

        Finestres.FiltresPressupostos filtres = new Finestres.FiltresPressupostos();

        /// <summary>
        /// Constructor de la classe
        /// </summary>
        public MantPressupostos(Usuari _usuari)
        {
            InitializeComponent();
            usuariActual = _usuari;
        }

        /// <summary>
        /// Funció encarregada de la càrrega dels usuaris donats d'alta al sistema.
        /// </summary>
        private void LoadPressupostos()
        {
            switch (filtres.filtreSeleccionat)
            {
                case Finestres.FiltresPressupostos.TipusFiltre.usuari:
                    pressupostos = client.getPressupostosByUser(usuariActual).ToList();
                    break;
                case Finestres.FiltresPressupostos.TipusFiltre.dates:
                    pressupostos = client.getPressupostosByDate(filtres.dataIni, filtres.dataFin).ToList();
                    break;
                case Finestres.FiltresPressupostos.TipusFiltre.acceptat:
                    pressupostos = client.getPressupostosByEstat(filtres.CertFals).ToList();
                    break;
                case Finestres.FiltresPressupostos.TipusFiltre.factura:
                    pressupostos = client.getPressupostosByFacturat(filtres.CertFals).ToList();
                    break;
            }
            
            ListGrid.ItemsSource = pressupostos;
        }

        /// <summary>
        /// Funció que salta qual el control s'ha carregat.
        /// </summary>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.LoadPressupostos();
        }

        /// <summary>
        /// Funció que salta qual es selecciona una fila del grid de pantalla.
        /// S'encarrega de carregar l'objecte de treball actual.
        /// </summary>
        private void ListGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;

            _pressupostActual = (Pressupost)e.AddedItems[0];
        }
        
        /// <summary>
        /// Funció encarregada de crear un nou element amb el que treballar i donar d'alta a la base de dades.
        /// </summary>
        private void b_nou_Click(object sender, RoutedEventArgs e)
        {
            _pressupostActual = new Pressupost();
            _pressupostActual.data = DateTime.Today;
            this.MostraPressupost();

        }

        /// <summary>
        /// Funció encarregada de mostrar a l'usuari el pressupost seleccionat.
        /// </summary>
        private void b_modificar_Click(object sender, RoutedEventArgs e)
        {
            if (_pressupostActual != null)
            {
                this.MostraPressupost();
            }
        }

        private void b_filtrar_Click(object sender, RoutedEventArgs e)
        {
            filtres.ShowDialog();
            this.LoadPressupostos();
        }

        private void MostraPressupost()
        {
            DetallPressupost detall = new DetallPressupost(usuariActual, _pressupostActual);
            ((Grid)this.Parent).Children.Add(detall);
            this.Visibility = System.Windows.Visibility.Collapsed;
        }

    }
}
