﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CentralMudances.Servei;

namespace CentralMudances.Controls
{
    /// <summary>
    /// Lógica de interacción para MantUsuaris.xaml
    /// </summary>
    public partial class MantUsuaris : UserControl
    {
        private List<Usuari> usuaris = new List<Usuari>();
        Servei.ServeiMudancesClient client = new Servei.ServeiMudancesClient();

        private Usuari _usuariActual = null;

        /// <summary>
        /// Constructor de la classe
        /// </summary>
        public MantUsuaris()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Funció encarregada de la càrrega dels usuaris donats d'alta al sistema.
        /// </summary>
        private void LoadUsers()
        {
            usuaris = client.getUsers().ToList();
            ListGrid.ItemsSource = usuaris;
        }

        /// <summary>
        /// Funció que salta qual el control s'ha carregat.
        /// </summary>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.LoadUsers();
        }

        /// <summary>
        /// Funció que salta qual es selecciona una fila del grid de pantalla.
        /// S'encarrega de carregar l'objecte de treball actual.
        /// </summary>
        private void ListGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;

            _usuariActual = (Usuari)e.AddedItems[0];
            this.MostraElementActual();

        }
        
        /// <summary>
        /// Funció encarregada de buidar tots els elements d'entrada de dades del formulari.
        /// </summary>
        private void BuidaFormulari()
        {
            t_nom.Text = "";
            t_password.Password = "";
            ck_admin.IsChecked = false;
            ck_baixa.IsChecked = false;
            ck_central.IsChecked = false;
            ck_wp8.IsChecked = false;
        }
        /// <summary>
        /// Funció encarregada d'emplenar els elements d'entrada de dades amb les dades de l'element actualment seleccionat.
        /// </summary>
        private void MostraElementActual()
        {
            this.BuidaFormulari();
            t_nom.Text = _usuariActual.nom;
            t_password.Password = _usuariActual.password;
            ck_admin.IsChecked = _usuariActual.administrador;
            ck_baixa.IsChecked = _usuariActual.baixa;
            ck_central.IsChecked = _usuariActual.central;
            ck_wp8.IsChecked = _usuariActual.wp8;
        }
        /// <summary>
        /// Funció encarrega d'actualitzar l'objecte amb els que estem treballant amb le dades que l'usuari ha introduït.
        /// </summary>
        private void ActualitzaElementActual()
        {
            _usuariActual.nom = t_nom.Text;
            _usuariActual.password = t_password.Password;
            _usuariActual.administrador = ck_admin.IsChecked.Value;
            _usuariActual.baixa = ck_baixa.IsChecked.Value;
            _usuariActual.central = ck_central.IsChecked.Value;
            _usuariActual.wp8 = ck_wp8.IsChecked.Value;
        }

        /// <summary>
        /// Funció encarregada de guardar les dades introduïdes per l'usuari.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void b_guarda_Click(object sender, RoutedEventArgs e)
        {
            if (_usuariActual == null)
                return;

            this.ActualitzaElementActual();
            if (_usuariActual.Id == 0)
            {
                client.addUser(_usuariActual);
            }
            else
            {
                client.updateUser(_usuariActual);
            }
            this.BuidaFormulari();
            this.LoadUsers();
        }
        /// <summary>
        /// Funció encarregada de crear un nou element amb el que treballar i donar d'alta a la base de dades.
        /// </summary>
        private void b_nou_Click(object sender, RoutedEventArgs e)
        {
            this.BuidaFormulari();
            _usuariActual = new Usuari();
        }

    }
}
