﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using CentralMudances.Servei;

namespace CentralMudances.Controls
{
    /// <summary>
    /// Lógica de interacción para MantUsuaris.xaml
    /// </summary>
    public partial class MantVehicles : UserControl
    {
        private List<Vehicle> vehicles = new List<Vehicle>();
        Servei.ServeiMudancesClient client = new Servei.ServeiMudancesClient();

        private Vehicle _vehicleActual = null;

        /// <summary>
        /// Constructor de la classe
        /// </summary>
        public MantVehicles()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Funció encarregada de la càrrega dels vehicles donats d'alta al sistema.
        /// </summary>
        private void LoadUsers()
        {
            vehicles = client.getVehicles().ToList();
            ListGrid.ItemsSource = vehicles;
        }

        /// <summary>
        /// Funció que salta qual el control s'ha carregat.
        /// </summary>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            this.LoadUsers();
        }

        /// <summary>
        /// Funció que salta qual es selecciona una fila del grid de pantalla.
        /// S'encarrega de carregar l'objecte de treball actual.
        /// </summary>
        private void ListGrid_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (e.AddedItems.Count == 0)
                return;

            _vehicleActual = (Vehicle)e.AddedItems[0];
            this.MostraElementActual();

        }
        
        /// <summary>
        /// Funció encarregada de buidar tots els elements d'entrada de dades del formulari.
        /// </summary>
        private void BuidaFormulari()
        {
            t_nom.Text = "";
            t_descripcio.Text = "";
            ck_baixa.IsChecked = false;
        }
        /// <summary>
        /// Funció encarregada d'emplenar els elements d'entrada de dades amb les dades de l'element actualment seleccionat.
        /// </summary>
        private void MostraElementActual()
        {
            this.BuidaFormulari();
            t_nom.Text = _vehicleActual.nom;
            t_descripcio.Text = _vehicleActual.descripcio;
            ck_baixa.IsChecked = _vehicleActual.baixa;
        }
        /// <summary>
        /// Funció encarrega d'actualitzar l'objecte amb els que estem treballant amb le dades que l'usuari ha introduït.
        /// </summary>
        private void ActualitzaElementActual()
        {
            _vehicleActual.nom = t_nom.Text;
            _vehicleActual.descripcio = t_descripcio.Text;
            _vehicleActual.baixa = ck_baixa.IsChecked.Value;
        }

        /// <summary>
        /// Funció encarregada de guardar les dades introduïdes per l'usuari.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void b_guarda_Click(object sender, RoutedEventArgs e)
        {
            if (_vehicleActual == null)
                return;

            this.ActualitzaElementActual();
            if (_vehicleActual.Id == 0)
            {
                client.addVehicle(_vehicleActual);
            }
            else
            {
                client.updateVehicle(_vehicleActual);
            }
            this.BuidaFormulari();
            this.LoadUsers();
        }
        /// <summary>
        /// Funció encarregada de crear un nou element amb el que treballar i donar d'alta a la base de dades.
        /// </summary>
        private void b_nou_Click(object sender, RoutedEventArgs e)
        {
            this.BuidaFormulari();
            _vehicleActual = new Vehicle();
        }

    }
}
