﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using Microsoft.Reporting.WinForms;
using System.Net;
using System.IO;

namespace CentralMudances.Finestres
{
    /// <summary>
    /// Lógica de interacción para ShowReport.xaml
    /// </summary>
    public partial class ShowReport : Window
    {
        Servei.ServeiMudancesClient client = new Servei.ServeiMudancesClient();
        List<Servei.Configuracio> configuracio;

        byte[] informePDF;

        /// <summary>
        /// Variables que farem servir per a passar paràmetres al formulari.
        /// </summary>
        public int ID;
        public bool Factura;
        public string eMailDestí = "";

        /// <summary>
        /// Constructor de la classe.
        /// </summary>
        public ShowReport()
        {
            InitializeComponent();
            _reportViewer.Load += _reportViewer_Load;
            this.LoadConfig();
        }
        /// <summary>
        /// Funció encarregada de carregar la configuració de la base de dades.
        /// </summary>
        private void LoadConfig()
        {
            configuracio = client.getConfiguracio().ToList();
        }
        /// <summary>
        /// Funció encarregada de retornar el valor del paràmetre indicat al paràmetre.
        /// </summary>
        private string GetConfigParameter(string parametre)
        {
            Servei.Configuracio cnf = (from i in configuracio
                                       where i.propietat == parametre
                                       select i).FirstOrDefault();

            if (cnf != null)
                return cnf.valor;
            else
                return "";
        }

        /// <summary>
        /// Funció encarregada de mostrar l'informe demanat, i si s'escau, enviar-lo per correu.
        /// </summary>
        void _reportViewer_Load(object sender, EventArgs e)
        {
            _reportViewer.ProcessingMode = Microsoft.Reporting.WinForms.ProcessingMode.Remote;
            _reportViewer.Enabled = true;

            ServerReport sr = _reportViewer.ServerReport;
            sr.ReportServerUrl = new Uri(GetConfigParameter("reportServer"));
            sr.ReportPath = GetConfigParameter((Factura ? "reportFactura" : "reportPressupost"));
            NetworkCredential nc = new NetworkCredential();
            nc.UserName = GetConfigParameter("reportUser");
            nc.Password = GetConfigParameter("reportPassword");
            sr.ReportServerCredentials.NetworkCredentials = nc;
            
            _reportViewer.ServerReport.SetParameters(new ReportParameter[] { new ReportParameter("Id", ID.ToString()) });

            informePDF = _reportViewer.ServerReport.Render("PDF");

            _reportViewer.RefreshReport();
        }

        /// <summary>
        /// Funció encarregada d'enviar l'informe mostrar per email al client.
        /// </summary>
        private void sendEmail_Click(object sender, RoutedEventArgs e)
        {
            string fitxer;
            string rutaFitxer;

            if (Factura)
                fitxer = "Factura" + ID.ToString().Trim() + ".pdf";
            else
                fitxer = "Albara" + ID.ToString().Trim() + ".pdf";

            rutaFitxer = System.IO.Path.Combine(System.IO.Path.GetTempPath(), fitxer);

            FileStream fs = File.Create(rutaFitxer);
            BinaryWriter bw = new BinaryWriter(fs);
            bw.Write(informePDF);
            bw.Close();
            fs.Close();

            EMail email = new EMail();
            if (email.SendEmail(eMailDestí, "MUDANCES: " + fitxer, "Adjuntem document", rutaFitxer))
            {
                MessageBox.Show("Correu enviat correctament", "Confirmació", MessageBoxButton.OK);
            }
            else
            {
                MessageBox.Show("Error enviant el correu electrònic.\nSi us plau, comprovi la configuració.", "Confirmació", MessageBoxButton.OK);
            }

        }

        /// <summary>
        /// Funció que salta quan el formulari s'ha mostrat per pantalla.
        /// Si no hem informat el correu electrònic deshabilita el botó que permetria enviar-lo.
        /// </summary>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (string.IsNullOrEmpty(eMailDestí))
                sendEmail.IsEnabled = false;
        }

    }
}
