﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace CentralMudances
{
    /// <summary>
    /// Lógica de interacción para Principal.xaml
    /// </summary>
    public partial class Principal : Window
    {
        private Servei.Usuari usuariActual;
        
        /// <summary>
        /// Constructor de la classe. 
        /// Reb l'usuari logat com a paràmetre.
        /// </summary>
        /// <param name="_usuariActual"></param>
        public Principal(Servei.Usuari _usuariActual)
        {
            InitializeComponent();

            this.usuariActual = _usuariActual;
        }

        /// <summary>
        /// Funció que s'executa quan el formulari ja s'ha carregat.
        /// </summary>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            this.AssignaPermisos();
        }

        /// <summary>
        /// Funció encarregada d'habilitar i deshabilitar els menús en funció dels permisos dels usuaris.
        /// </summary>
        private void AssignaPermisos()
        {
            if (usuariActual.administrador)
                return;

            if (usuariActual.central)
            {
                mConfiguracio.IsEnabled = false;
                mUsuaris.IsEnabled = false;
            }

            if (usuariActual.wp8)
            {
                mConfiguracio.IsEnabled = false;
                mUsuaris.IsEnabled = false;
                mOperaris.IsEnabled = false;
                mClients.IsEnabled = false;
                mVehicles.IsEnabled = false;
            }
        }

        #region "Funcions de menús encarregades de crear l'objecte de manteniment que es mostrarà en pantalla"
        private void mUsuaris_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormUsuaris();
        }
        private void iUsuaris_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormUsuaris();
        }
        private void CarregaFormUsuaris()
        {
            Controls.MantUsuaris frmUsuari = new Controls.MantUsuaris();
            this.CarregaControl(frmUsuari);
        }

        private void mOperaris_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormOperaris();
        }
        private void iOperaris_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormOperaris();
        }
        private void CarregaFormOperaris()
        {
            Controls.MantOperaris frmOperari = new Controls.MantOperaris();
            this.CarregaControl(frmOperari);
        }

        private void mClients_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormClients();
        }
        private void iClients_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormClients();
        }
        private void CarregaFormClients()
        {
            Controls.MantClients frmClient = new Controls.MantClients();
            this.CarregaControl(frmClient);
        }

        private void mVehicles_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormVehicles();
        }
        private void iVehicles_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormVehicles();
        }
        private void CarregaFormVehicles()
        {
            Controls.MantVehicles frmVehicles = new Controls.MantVehicles();
            this.CarregaControl(frmVehicles);
        }

        private void mPressupostos_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormPressupostos();
        }
        private void iPressupostos_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormPressupostos();
        }
        private void CarregaFormPressupostos()
        {
            Controls.MantPressupostos frmPressupostos = new Controls.MantPressupostos(usuariActual);
            this.CarregaControl(frmPressupostos);
        }

        private void mConfiguracio_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormConfiguracio();

        }
        private void iConfig_Click(object sender, RoutedEventArgs e)
        {
            this.CarregaFormConfiguracio();
        }
        private void CarregaFormConfiguracio()
        {
            Controls.MantConfiguracio frmConfiguracio = new Controls.MantConfiguracio();
            this.CarregaControl(frmConfiguracio);
        }
        #endregion

        /// <summary>
        /// Funció encarregada d'afegir el control passat a la plana principal.
        /// </summary>
        private void CarregaControl(UIElement element)
        {
            GridContenidor.Children.Clear();
            GridContenidor.Children.Add(element);
        }

    }
}
