﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace Mudances_WCF
{
    public class ConfiguracioDAO
    {
        MudancesEntities bdMudances;

        /// <summary>
        /// Constructor de la classe.
        /// </summary>
        public ConfiguracioDAO()
        {
            bdMudances = new MudancesEntities();
        }

        /// <summary>
        /// Retorna la llista de la configuració creada a la base de dades.
        /// </summary>
        /// <returns>Un objecte IEnumerable amb tota la configuració creada.</returns>
        public List<Configuracio> getConfiguracio()
        {
            List<Configuracio> configuracio = new List<Configuracio>();
            try
            {
                IEnumerable<configuracioSet> temp = from i in bdMudances.configuracioSet
                                                    select i;

                foreach (configuracioSet cnf in temp)
                {
                    configuracio.Add(new Configuracio(cnf));
                }
            }
            catch
            { /* Do nothign */ }

            return configuracio;
        }

        /// <summary>
        /// Funció encarregada de crear un nou registre de configuració a la base de dades
        /// </summary>
        /// <param name="cli">Instància de l'objecte que es crearà a la base de dades</param>
        /// <returns>Verdader si s'ha pogut crear l'objecte, Fals en cas contrari</returns>
        public bool addConfiguracio(Configuracio cnf)
        {
            try
            {
                configuracioSet newConf = new configuracioSet();
                this.PocoToDAO(ref newConf, ref cnf);
                bdMudances.configuracioSet.Add(newConf);
                bdMudances.SaveChanges();
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Funció encarregada d'actualitzar les dades d'un registre de configuració a la base de dades
        /// </summary>
        /// <param name="cli">Instància de l'objecte modificat</param>
        /// <returns>Verdader si s'ha pogut actualizar l'objecte, Fals en cas contrari</returns>
        public bool updateConfiguracio(Configuracio cnf)
        {
            try
            {
                configuracioSet old = (from i in bdMudances.configuracioSet
                                       where i.Id == cnf.Id
                                       select i).First();

                this.PocoToDAO(ref old, ref cnf);

                bdMudances.SaveChanges();
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Funció encarregada de la transformació d'una entitat POCO a una del tipus DAO.
        /// </summary>
        private void PocoToDAO(ref configuracioSet DAO, ref Configuracio POCO)
        {
            DAO.propietat = POCO.propietat;
            DAO.valor = POCO.valor;
        }
    }
}