﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace Mudances_WCF
{
    public class OperarisDAO
    {
        MudancesEntities bdMudances;
        
        /// <summary>
        /// Constructor de la classe
        /// </summary>
        public OperarisDAO()
        {
            bdMudances = new MudancesEntities();
        }

        /// <summary>
        /// Retorna la llista de tots els operaris creats a la base de dades.
        /// </summary>
        /// <returns>Un objecte IEnumerable amb tots el operaris creats.</returns>
        public List<Operari> getOperaris()
        {
            List<Operari> operaris = new List<Operari>();
            
            try
            {
                IEnumerable<operariSet> temp = from i in bdMudances.operariSet
                                               select i;

                foreach (operariSet op in temp)
                {
                    operaris.Add(new Operari(op));
                }
            }
            catch
            { /* Do nothing */ }
            
            return operaris;
        }

        /// <summary>
        /// Funció encarregada de crear un nou operari a la base de dades
        /// </summary>
        /// <param name="op">Instància de l'objecte que es crearà a la base de dades</param>
        /// <returns>Verdader si s'ha pogut crear l'objecte, Fals en cas contrari</returns>
        public bool addOperari(Operari op)
        {
            try
            {
                operariSet newOperari = new operariSet();
                this.PocoToDAO(ref newOperari, ref op);
                bdMudances.operariSet.Add(newOperari);
                bdMudances.SaveChanges();
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Funció encarregada d'actualitzar les dades d'un operari a la base de dades
        /// </summary>
        /// <param name="op">Instància de l'objecte modificat</param>
        /// <returns>Verdader si s'ha pogut actualizar l'objecte, Fals en cas contrari</returns>
        public bool updateOperari(Operari op)
        {
            try
            {
                operariSet old = (from i in bdMudances.operariSet
                                  where i.Id == op.Id
                                  select i).First();

                this.PocoToDAO(ref old, ref op);

                bdMudances.SaveChanges();

            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Funció encarregada de la transformació d'una entitat POCO a una del tipus DAO.
        /// </summary>
        private void PocoToDAO(ref operariSet DAO, ref Operari POCO)
        {
            DAO.nom = POCO.nom;
            DAO.comentaris = POCO.comentaris;
            DAO.baixa = POCO.baixa;
        }
    }
}