﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace Mudances_WCF
{
    public class PressupostosDAO
    {
        MudancesEntities bdMudances;

        /// <summary>
        /// Constructor de la classe.
        /// </summary>
        public PressupostosDAO()
        {
            bdMudances = new MudancesEntities();
            bdMudances.Configuration.LazyLoadingEnabled = true;
        }

        /// <summary>
        /// Funció encarregada de retornar la llista de pressupostos transformats de DAO a POCO.
        /// </summary>
        public List<Pressupost> getPressupostos()
        {
            List<Pressupost> pressupostos = new List<Pressupost>();
            try
            {
                IEnumerable<pressupostSet> temp = from i in bdMudances.pressupostSet
                                                  select i;

                foreach (pressupostSet pre in temp)
                {
                    pressupostos.Add(new Pressupost(pre));
                }
            }
            catch
            { /* Do nothign */ }
            
            return pressupostos;
        }

        /// <summary>
        /// Funció encarregada de retornar la llista de pressupostos transformats de DAO a POCO que corresponen a 
        /// l'usuari informat al paràmetre.
        /// </summary>
        public List<Pressupost> getPressupostos(Usuari user)
        {
            List<Pressupost> pressupostos = new List<Pressupost>();
            try
            {
                IEnumerable<pressupostSet> temp = from i in bdMudances.pressupostSet
                                                  where i.usuari_Id == user.Id
                                                  select i;

                foreach (pressupostSet pre in temp)
                {
                    pressupostos.Add(new Pressupost(pre));
                }
            }
            catch
            { /* Do nothign */ }

            return pressupostos;

        }

        /// <summary>
        /// Funció encarregada de retornar la llista de pressupostos transformats de DAO a POCO que es van crear
        /// entre les dates informades als paràmetres.
        /// </summary>
        public List<Pressupost> getPressupostos(DateTime dIni, DateTime dFin)
        {
            List<Pressupost> pressupostos = new List<Pressupost>();
            try
            {
                IEnumerable<pressupostSet> temp = from i in bdMudances.pressupostSet
                                                  where i.data >= dIni
                                                  && i.data <= dFin
                                                  select i;

                foreach (pressupostSet pre in temp)
                {
                    pressupostos.Add(new Pressupost(pre));
                }
            }
            catch
            { /* Do nothign */ }

            return pressupostos;

        }

        /// <summary>
        /// Funció encarregada de retornar la llista de pressupostos transformats de DAO a POCO que estan acceptats
        /// o no en funció del paràmetre informat.
        /// </summary>
        public List<Pressupost> getPressupostosByEstat(bool acceptat)
        {
            List<Pressupost> pressupostos = new List<Pressupost>();
            try
            {
                IEnumerable<pressupostSet> temp = from i in bdMudances.pressupostSet
                                                  where i.acceptat == acceptat
                                                  select i;

                foreach (pressupostSet pre in temp)
                {
                    pressupostos.Add(new Pressupost(pre));
                }
            }
            catch
            { /* Do nothign */ }

            return pressupostos;
        }

        /// <summary>
        /// Funció encarregada de retornar la llista de pressupostos transformats de DAO a POCO que estan facturats
        /// o no en funció del paràmetre informat.
        /// </summary>
        public List<Pressupost> getPressupostByFacturat(bool facturat)
        {
            List<Pressupost> pressupostos = new List<Pressupost>();
            try
            {
                IEnumerable<pressupostSet> temp = from i in bdMudances.pressupostSet
                                                  where i.facturat == facturat
                                                  select i;

                foreach (pressupostSet pre in temp)
                {
                    pressupostos.Add(new Pressupost(pre));
                }
            }
            catch
            { /* Do nothign */ }

            return pressupostos;

        }

        /// <summary>
        /// Funció encarregada de retornar el pressupost corresponent a l'Id passat transformat de DAO a POCO.
        /// </summary>
        public Pressupost getPressupostById(int id)
        {
            try
            {
                pressupostSet temp = (from i in bdMudances.pressupostSet
                                      where i.Id == id
                                      select i).FirstOrDefault();

                if (temp != null)
                {
                    return new Pressupost(temp);
                }
            }
            catch
            { /* Do nothing */ }

            return null;
        }

        /// <summary>
        /// Funció encarregada d'afegir a la base de dades el pressupost passat en format POCO.
        /// </summary>
        public bool addPressupost(Pressupost pressupost)
        {
            try
            {
                pressupostSet nouPressupost = new pressupostSet();
                this.PocoToDAO(ref nouPressupost, ref pressupost);
                bdMudances.pressupostSet.Add(nouPressupost);
                bdMudances.SaveChanges();
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Funció encarregada d'actualitzar el pressupost passat en format POCO a la base de dades.
        /// </summary>
        public bool updatePressupost(Pressupost pressupost)
        {
            try
            {
                var old = (from i in bdMudances.pressupostSet
                           where i.Id == pressupost.Id
                           select i).FirstOrDefault();

                this.PocoToDAO(ref old, ref pressupost);
                bdMudances.SaveChanges();
            }
            catch
            {
                return false;
            }

            return true;

        }

        /// <summary>
        /// Funció encarregada de facturar el pressupost passat.
        /// </summary>
        public bool facturaPressupost(int Id)
        {
            try
            {
                int numFactura;

                var maxFact = (from i in bdMudances.pressupostSet
                               where i.facturat == true
                               orderby i.num_factura descending
                               select i.num_factura).FirstOrDefault();


                if (maxFact == null)
                    numFactura = 1;
                else
                    numFactura = maxFact + 1;

                Pressupost temp = getPressupostById(Id);
                temp.facturat = true;
                temp.num_factura = numFactura;
                updatePressupost(temp);
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Funció encarregada de la transformació d'una entitat POCO a una del tipus DAO.
        /// </summary>
        private void PocoToDAO(ref pressupostSet DAO, ref Pressupost POCO)
        {
            DAO.data = POCO.data;
            DAO.acceptat = POCO.acceptat;
            DAO.facturat = POCO.facturat;
            DAO.num_factura = POCO.num_factura;
            DAO.adreça_origen = POCO.adreça_origen;
            DAO.adreça_desti = POCO.adreça_desti;
            DAO.cp_origen = POCO.cp_origen;
            DAO.cp_desti = POCO.cp_desti;
            DAO.poblacio_origen = POCO.poblacio_origen;
            DAO.poblacio_desti = POCO.poblacio_desti;
            DAO.provincia_origen = POCO.provincia_origen;
            DAO.provincia_desti = POCO.provincia_desti;
            DAO.usuari_Id = POCO.usuari.Id;
            DAO.client_Id = POCO.client.Id;

            //Assignem els operaris que l'usuari ha assignat al pressupost.
            if (POCO.operaris == null || POCO.operaris.Count == 0)
            {
                if (DAO.operariSet != null)
                    DAO.operariSet.Clear();
            }
            else
            {
                List<int> operarisID = (from i in POCO.operaris select i.Id).ToList();

                IEnumerable<operariSet> operaris = from i in bdMudances.operariSet
                                                   where operarisID.Contains(i.Id)
                                                   select i;

                DAO.operariSet.Clear();
                foreach (operariSet operari in operaris)
                {
                    DAO.operariSet.Add(operari);
                }
            }

            //Assignem els vehicles que l'usuari ha assignat al pressupost.
            if (POCO.vehicles == null || POCO.vehicles.Count == 0)
            {
                if (DAO.vehicleSet != null)
                    DAO.vehicleSet.Clear();
            }
            else
            {
                List<int> vehiclesId = (from i in POCO.vehicles select i.Id).ToList();

                IEnumerable<vehicleSet> vehicles = from i in bdMudances.vehicleSet
                                                   where vehiclesId.Contains(i.Id)
                                                   select i;

                DAO.vehicleSet.Clear();
                foreach (vehicleSet vehicle in vehicles)
                {
                    DAO.vehicleSet.Add(vehicle);
                }
                
            }

            //Si no hi ha linies al pressupost esborrem totes les que hi puguin haber.
            if (POCO.linies == null || POCO.linies.Count == 0)
            {
                if (DAO.linia_pressupostSet != null)
                {
                    DAO.linia_pressupostSet.Clear();
                }
            }
            else
            {
                List<int> liniesId = (from i in POCO.linies select i.Id).ToList();
                
                //Esborrem i actualitzem les línies corresponents
                if (DAO.linia_pressupostSet != null)
                {
                    //Esborrem les línies que ja no existeixin.
                    List<linia_pressupostSet> liniesBorrar = (from i in DAO.linia_pressupostSet
                                                             where !liniesId.Contains(i.Id)
                                                             select i).ToList();

                    for (int i = 0; i < liniesBorrar.Count(); i++)
                    {
                        DAO.linia_pressupostSet.Remove(liniesBorrar[i]);
                    }

                    //Actualitzem les línies que ja existeixen.
                    //No podem fer servir un IEnumerable perquè no permet la modificació del elements d'enumeració
                    //d'un bucle.
                    List<int> liniesActualitzar = (from i in DAO.linia_pressupostSet
                                                   where liniesId.Contains(i.Id)
                                                   select i.Id).ToList();

                    LiniaPressupostDAO liniaPressupostDAO = new LiniaPressupostDAO();
                    foreach (int liniaActualitzar in liniesActualitzar)
                    {
                        linia_pressupostSet tempDAO = DAO.linia_pressupostSet.First(c => c.Id == liniaActualitzar);
                        LiniaPressupost tempPOCO = POCO.linies.First(c => c.Id == liniaActualitzar);
                        liniaPressupostDAO.POCOtoDAO(ref tempDAO, tempPOCO);
                        tempDAO.imatgesSet.Clear();
                        foreach (Imatge imatge in tempPOCO.imatges)
                        {
                            imatgesSet nova = new imatgesSet();
                            nova.imatge = imatge.imatge;
                            tempDAO.imatgesSet.Add(nova);
                        }
                    }

                    //Creem les línies noves.
                    IEnumerable<LiniaPressupost> novesLinies = from i in POCO.linies
                                                               where i.Id <= 0
                                                               select i;

                    foreach (LiniaPressupost novaLinia in novesLinies)
                    {
                        linia_pressupostSet nova = new linia_pressupostSet();
                        liniaPressupostDAO.POCOtoDAO(ref nova, novaLinia);
                        DAO.linia_pressupostSet.Add(nova);

                        if (novaLinia.imatges != null)
                            foreach (Imatge imatge in novaLinia.imatges)
                            {
                                imatgesSet novaImatge = new imatgesSet();
                                novaImatge.imatge = imatge.imatge;
                                nova.imatgesSet.Add(novaImatge);
                            }
                    }
                }

            }
        }
    }
}