﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace Mudances_WCF
{
    public class UsuarisDAO
    {
        MudancesEntities bdMudances;

        /// <summary>
        /// Constructor de la classe
        /// </summary>
        public UsuarisDAO()
        {
            bdMudances = new MudancesEntities();
        }

        /// <summary>
        /// Retorna la llista de tots els usuaris creats a la base de dades.
        /// </summary>
        /// <returns>Un objecte IEnumerable amb tots el usuaris creats.</returns>
        public List<Usuari> getUsers()
        {
            List<Usuari> usuaris = new List<Usuari>();
            try
            {
                IEnumerable<usuariSet> temp = from i in bdMudances.usuariSet
                                              select i;

                foreach (var usr in temp)
                {
                    usuaris.Add(new Usuari(usr));
                }

                return usuaris;

            }
            catch
            {
                return usuaris;
            }
        }

        /// <summary>
        /// Funció encarregada de crear un nou usuari a la base de dades
        /// </summary>
        /// <param name="user">Instància de l'objecte que es crearà a la base de dades</param>
        /// <returns>Verdader si s'ha pogut crear l'objecte, Fals en cas contrari</returns>
        public bool addUser(Usuari user)
        {
            try
            {
                usuariSet nouUsuari = new usuariSet();
                this.PocoToDAO(ref nouUsuari, ref user);
                bdMudances.usuariSet.Add(nouUsuari);
                bdMudances.SaveChanges();
            }
            catch
            {
                return false;
            }
            
            return true;

        }

        /// <summary>
        /// Funció encarregada d'actualitzar les dades d'un usuari a la base de dades
        /// </summary>
        /// <param name="user">Instància de l'objecte modificat</param>
        /// <returns>Verdader si s'ha pogut actualizar l'objecte, Fals en cas contrari</returns>
        public bool updateUser(Usuari user)
        {
            try
            {
                var old = (from i in bdMudances.usuariSet
                          where i.Id == user.Id
                          select i).First();

                this.PocoToDAO(ref old, ref user);
                bdMudances.SaveChanges();
                
            }
            catch
            {
                return false;
            }
         
            return true;

        }

        /// <summary>
        /// Funció encarregada d'autenticar un usuari a la base de dades.
        /// </summary>
        /// <param name="name">Nom de l'usuari introduït</param>
        /// <param name="pwd">Contrassenya de l'ususari introduïda</param>
        /// <returns>Una instància de l'objecte usuari definit, o NULL en cas de no trobar-lo.</returns>
        public Usuari validaUsuari(string name, string pwd)
        {
            try
            {
                usuariSet temp = (from i in bdMudances.usuariSet
                                  where i.nom.Trim() == name.Trim()
                                  && i.password.Trim() == pwd.Trim()
                                  select i).FirstOrDefault();

                if (temp != null)
                {
                    return new Usuari(temp);
                }
            }
            catch
            { /* Do nothing */ }

            return null;
        }

        /// <summary>
        /// Funció encarregada de la transformació d'una entitat POCO a una del tipus DAO.
        /// </summary>
        private void PocoToDAO(ref usuariSet DAO, ref Usuari POCO)
        {
            DAO.administrador = POCO.administrador;
            DAO.baixa = POCO.baixa;
            DAO.central = POCO.central;
            DAO.nom = POCO.nom;
            DAO.password = POCO.password;
            DAO.wp8 = POCO.wp8;
        }
    }
}