﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;

namespace Mudances_WCF
{
    public class VehiclesDAO
    {
        MudancesEntities bdMudances;

        /// <summary>
        /// Constructor de la classe.
        /// </summary>
        public VehiclesDAO()
        {
            bdMudances = new MudancesEntities();
        }

        /// <summary>
        /// Retorna la llista de tots els vehicles creats a la base de dades.
        /// </summary>
        /// <returns>Un objecte IEnumerable amb tots el vehicles creats.</returns>
        public List<Vehicle> getVehicles()
        {
            List<Vehicle> vehicles = new List<Vehicle>();
            try
            {
                IEnumerable<vehicleSet> temp = from i in bdMudances.vehicleSet
                                               select i;

                foreach (vehicleSet veh in temp)
                {
                    vehicles.Add(new Vehicle(veh));
                }
            }
            catch
            { /* Do nothing */ }

            return vehicles;
        }

        /// <summary>
        /// Funció encarregada de crear un nou vehicle a la base de dades
        /// </summary>
        /// <param name="veh">Instància de l'objecte que es crearà a la base de dades</param>
        /// <returns>Verdader si s'ha pogut crear l'objecte, Fals en cas contrari</returns>
        public bool addVehicle(Vehicle veh)
        {
            try
            {
                vehicleSet newVehicle = new vehicleSet();
                this.PocoToDAO(ref newVehicle, ref veh);
                bdMudances.vehicleSet.Add(newVehicle);
                bdMudances.SaveChanges();
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Funció encarregada d'actualitzar les dades d'un vehicle a la base de dades
        /// </summary>
        /// <param name="veh">Instància de l'objecte modificat</param>
        /// <returns>Verdader si s'ha pogut actualizar l'objecte, Fals en cas contrari</returns>
        public bool updateVehicle(Vehicle veh)
        {
            try
            {
                vehicleSet old = (from i in bdMudances.vehicleSet
                                  where i.Id == veh.Id
                                  select i).First();

                this.PocoToDAO(ref old, ref veh);
                bdMudances.SaveChanges();
            }
            catch
            {
                return false;
            }

            return true;
        }

        /// <summary>
        /// Funció encarregada de la transformació d'una entitat POCO a una del tipus DAO.
        /// </summary>
        private void PocoToDAO(ref vehicleSet DAO, ref Vehicle POCO)
        {
            DAO.nom = POCO.nom;
            DAO.descripcio = POCO.descripcio;
            DAO.baixa = POCO.baixa;
        }
    }
}