﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Runtime.Serialization;

namespace Mudances_WCF
{
    /// <summary>
    /// Classe que es fa servir per a la comunicació a través del servei de WCF.
    /// </summary>
    [DataContract]
    public class Pressupost
    {
        /// <summary>
        /// Constructor de la classe.
        /// </summary>
        public Pressupost()
        {
            linies = new List<LiniaPressupost>();
            vehicles = new List<Vehicle>();
            operaris = new List<Operari>();
        }
        /// <summary>
        /// Constructor de la classe que rep com a paràmetre el seu objecte corresponent a la capa DAO.
        /// S'encarrega d'inicialitzar els valors en funció de l'objecte rebut.
        /// </summary>
        public Pressupost(pressupostSet dao)
        {
            linies = new List<LiniaPressupost>();
            vehicles = new List<Vehicle>();
            operaris = new List<Operari>();

            this.Id = dao.Id;
            this.data = dao.data;
            this.acceptat = dao.acceptat;
            this.facturat = dao.facturat;
            this.num_factura = dao.num_factura;
            this.adreça_origen = dao.adreça_origen;
            this.adreça_desti = dao.adreça_desti;
            this.cp_origen = dao.cp_origen;
            this.cp_desti = dao.cp_desti;
            this.poblacio_origen = dao.poblacio_origen;
            this.poblacio_desti = dao.poblacio_desti;
            this.provincia_origen = dao.provincia_origen;
            this.provincia_desti = dao.provincia_desti;

            usuari = new Usuari(dao.usuariSet);
            client = new Client(dao.clientSet);

            if (dao.vehicleSet.Count > 0)
            {
                foreach (vehicleSet vehicle in dao.vehicleSet)
                {
                    vehicles.Add(new Vehicle(vehicle));
                }
            }

            if (dao.operariSet.Count > 0)
            {
                foreach (operariSet operari in dao.operariSet)
                {
                    operaris.Add(new Operari(operari));
                }
            }

            if (dao.linia_pressupostSet.Count > 0)
            {
                foreach (linia_pressupostSet linia in dao.linia_pressupostSet)
                {
                    linies.Add(new LiniaPressupost(linia));
                }
            }
        }

        [DataMember]
        public int Id;
        [DataMember]
        public DateTime data;
        [DataMember]
        public bool acceptat;
        [DataMember]
        public bool facturat;
        [DataMember]
        public int num_factura;
        [DataMember]
        public string adreça_origen;
        [DataMember]
        public string adreça_desti;
        [DataMember]
        public string cp_origen;
        [DataMember]
        public string cp_desti;
        [DataMember]
        public string poblacio_origen;
        [DataMember]
        public string poblacio_desti;
        [DataMember]
        public string provincia_origen;
        [DataMember]
        public string provincia_desti;

        [DataMember]
        public Usuari usuari;
        [DataMember]
        public Client client;
        [DataMember]
        public List<Vehicle> vehicles;
        [DataMember]
        public List<Operari> operaris;
        [DataMember]
        public List<LiniaPressupost> linies;
    }
}