﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Tasks;
using System.IO;
using System.Windows.Media.Imaging;


namespace Mudances_WP8
{
    public partial class DetallLinia : PhoneApplicationPage
    {

        CameraCaptureTask cc = new CameraCaptureTask();

        /// <summary>
        /// Constructor de la classe.
        /// </summary>
        public DetallLinia()
        {
            InitializeComponent();

            cc.Completed += cc_Completed;
            this.ShowData();
        }

        /// <summary>
        /// Funció que salta quan s'ha creat una nova imatge.
        /// Aquesta s'afegeix a la línia actual.
        /// </summary>
        void cc_Completed(object sender, PhotoResult e)
        {
            if (e.TaskResult == TaskResult.OK)
            {
                WP_DAO.DataInterface di = new WP_DAO.DataInterface();

                WP_DAO.ServeiMudances.Imatge newImatge = new WP_DAO.ServeiMudances.Imatge();
                newImatge.Id = di.NumeroFoto();

                BinaryReader br = new BinaryReader(e.ChosenPhoto);
                newImatge.imatge = new byte[br.BaseStream.Length];
                br.Read(newImatge.imatge, 0, (int)br.BaseStream.Length);
                App.currentLiniaPressupost.imatges.Add(newImatge);
                App.pressupostModificat = true;

                this.ShowData();
            }
        }

        /// <summary>
        /// Funció encarregada de mostrar les dades de la línia en pantalla.
        /// </summary>
        private void ShowData()
        {
            SPImages.Children.Clear();

            if (!App.currentPressupost.acceptat)
            {
                ckTraslladat.Visibility = System.Windows.Visibility.Collapsed;
                tTemps.Visibility = System.Windows.Visibility.Collapsed;
                lTemps.Visibility = System.Windows.Visibility.Collapsed;
            }

            tArticle.Text = App.currentLiniaPressupost.article;
            tComentaris.Text = App.currentLiniaPressupost.comentaris;
            ckTraslladat.IsChecked = App.currentLiniaPressupost.transportat;
            tTemps.Text = App.currentLiniaPressupost.temps_dedicat.ToString();

            foreach (WP_DAO.ServeiMudances.Imatge imatge in App.currentLiniaPressupost.imatges)
            {
                MemoryStream ms = new MemoryStream(imatge.imatge);

                Image img = new Image();
                BitmapImage bi = new BitmapImage();
                bi.SetSource(ms);
                img.Height = 100;
                img.Width = 100;
                img.Source = bi;
                img.Margin = new Thickness(10);
                SPImages.Children.Add(img);
            }
        }

        /// <summary>
        /// Funció encarregada de mostrar l'aplicació de fotografies del terminal.
        /// </summary>
        private void ferFoto()
        {
            cc.Show();
        }

        /// <summary>
        /// Funció encarregada de capturar una nova imatge i afegir-la a la línia actual.
        /// </summary>
        private void AppBarPhoto_Click(object sender, EventArgs e)
        {
            this.ferFoto();
        }

        /// <summary>
        /// Funció encarregada d'anar enrere sense validar les dades.
        /// Es descarten els canvis.
        /// </summary>
        private void AppBarCancela_Click(object sender, EventArgs e)
        {
            NavigationService.GoBack();
        }

        /// <summary>
        /// Funció encarregada de guardar les dades introduïdes en cas de ser correctes i tornar enrere.
        /// </summary>
        private void AppBarOk_Click(object sender, EventArgs e)
        {
            App.pressupostModificat = true;
            App.currentLiniaPressupost.article = tArticle.Text;
            App.currentLiniaPressupost.comentaris = tComentaris.Text;
            App.currentLiniaPressupost.transportat = ckTraslladat.IsChecked.Value;
            
            if(App.currentPressupost.acceptat)
                App.currentLiniaPressupost.temps_dedicat = int.Parse(tTemps.Text);

            NavigationService.GoBack();
        }

        /// <summary>
        /// Funció encarregada de validar si les dades introduïdes a la línia son vàlides o no.
        /// </summary>
        private bool validaLinia()
        {
            int temp;

            ///Aquests camps no es veuen si el pressupost no està acceptat.
            if (!App.currentPressupost.acceptat)
                return true;

            if (string.IsNullOrEmpty(tTemps.Text))
            {
                MessageBox.Show("El temps dedicat no pot deixar-se en blanc.", "Error", MessageBoxButton.OK);
                return false;
            }
            
            if (!int.TryParse(tTemps.Text, out temp))
            {
                MessageBox.Show("El temps dedicat ha de ser un valor numèric", "Error", MessageBoxButton.OK);
                return false;
            }
            return true;
        }

    }
}