﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Mudances_WP8.Resources;
using Microsoft.Phone.Scheduler;

namespace Mudances_WP8
{
    public partial class MainPage : PhoneApplicationPage
    {
        WP_DAO.BaseDades.WP8_Configuration Configuracio;
        WP_DAO.DataInterface di = new WP_DAO.DataInterface();

        PeriodicTask periodicTask = null;
        string periodicTaskName = "AutoSYNC";

        /// <summary>
        /// Constructor de la classe.
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            this.inicialitzaBackgound();
        }

        private void inicialitzaBackgound()
        {
            periodicTask = ScheduledActionService.Find(periodicTaskName) as PeriodicTask;
            if (periodicTask == null)
            {
                periodicTask = new PeriodicTask(periodicTaskName);
                periodicTask.Description = "Mudances Sync";
                ScheduledActionService.Add(periodicTask);
#if(DEBUG)
            ScheduledActionService.LaunchForTest(periodicTaskName, TimeSpan.FromSeconds(60));
#endif
            }
        }

        /// <summary>
        /// Funció llançada al carregar la pantalla.
        /// S'encarrega d'anar a buscar la configuració i inicialitzar 
        /// els camps d'entrada amb els valor previament guardats.
        /// </summary>
        private void LayoutRoot_Loaded(object sender, RoutedEventArgs e)
        {
            Configuracio = di.CarregaConfiguracio();
            if (Configuracio != null)
            {
                tUser.Text = Configuracio.UserName;
                tPassword.Password = Configuracio.Password;
            }
            else
            {
                Configuracio = new WP_DAO.BaseDades.WP8_Configuration();
            }
        }

        /// <summary>
        /// Funció encarregada de validar l'usuari i actualitzar la configuració d'entrada.
        /// Si tot es correcte, obre la finesta amb la llista de pressupostos i mudances pendents.
        /// </summary>
        private void AppBar_OK_Click(object sender, EventArgs e)
        {
            WP_DAO.ServeiMudances.Usuari usuari = di.ValidaUsuari(tUser.Text, tPassword.Password);
            if (usuari == null)
            {
                MessageBox.Show("Usuari o Password INCORRECTES", "ERROR", MessageBoxButton.OK);
            }
            else
            {
                Configuracio.UserName = tUser.Text;
                Configuracio.Password = tPassword.Password;
                di.GuardaConfiguracio(Configuracio);
                App.currentUser = usuari;
                NavigationService.Navigate(new Uri("/PressupostosTrasllats.xaml", UriKind.Relative));
            }
        }

        /// <summary>
        /// Funció encarregada d'esborrar les dades de la pantalla inicial de l'aplicació.
        /// </summary>
        private void AppBar_Cancel_Click(object sender, EventArgs e)
        {
            tUser.Text = "";
            tPassword.Password = "";
        }

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            WP_DAO.Sync.Main a = new WP_DAO.Sync.Main();
            a.SyncAll();
        }

    }
}