﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WP_DAO.BaseDades;
using WP_DAO.Sync;

namespace WP_DAO
{
    /// <summary>
    /// Classe encarregada de la comunicació entre la llibreria lògica de WP8 i l'aplicació del terminal.
    /// </summary>
    public class DataInterface
    {
        /// <summary>
        /// Funció encarregada de retornar la configuració de l'aplicació guardada al dispositiu.
        /// </summary>
        public WP8_Configuration CarregaConfiguracio()
        {
            BaseDades.Main bd = new BaseDades.Main();
            return bd.CarregaConfiguracio();
        }

        /// <summary>
        /// Funció encarregada d'actualitzar la configuració guardada al dispositiu.
        /// </summary>
        public void GuardaConfiguracio(WP8_Configuration config)
        {
            BaseDades.Main bd = new BaseDades.Main();
            bd.GuardaConfiguracio(config);
        }

        /// <summary>
        /// Funció encarregada de validar l'usuari introduït.
        /// Si es correcte, retorna l'objecte usuari de la base de dades, en cas contrari retornarà un Null.
        /// </summary>
        public ServeiMudances.Usuari ValidaUsuari(string usuari, string password)
        {
            BaseDades.Main bd = new BaseDades.Main();
            WP8_Usuari user = bd.ValidaUsuari(usuari, password);
            if (user == null)
                return null;
            else
                return ConversorsBDServei.WP8ToServei(user);
        }

        /// <summary>
        /// Funció encarregada de retornar a l'usuari els pressupostos associats a l'usuari pendents d'acceptació´.
        /// </summary>
        public ObservableCollection<ServeiMudances.Pressupost> GetPressupostosUsuari(ServeiMudances.Usuari usuari)
        {
            ObservableCollection<ServeiMudances.Pressupost> pressupostos = new ObservableCollection<ServeiMudances.Pressupost>();

            BaseDades.Main bd = new BaseDades.Main();
            foreach (WP8_Pressupost pressupost in bd.GetPressupostosUsuari(usuari.Id))
            {
                pressupostos.Add(ConversorsBDServei.WP8ToServei(pressupost));
            }

            return pressupostos;

        }

        /// <summary>
        /// Funció encarregada de retornar a l'usuari els pressupostos associats a l'usuari i que ja estan acceptats.
        /// </summary>
        public ObservableCollection<ServeiMudances.Pressupost> GetTrasllatsUsuari(ServeiMudances.Usuari usuari)
        {
            ObservableCollection<ServeiMudances.Pressupost> trasllats = new ObservableCollection<ServeiMudances.Pressupost>();

            BaseDades.Main bd = new BaseDades.Main();
            foreach (WP8_Pressupost trasllat in bd.GetTrasllatsUsuari(usuari.Id))
            {
                trasllats.Add(ConversorsBDServei.WP8ToServei(trasllat));
            }

            return trasllats;

        }

        /// <summary>
        /// Funció encarregada d'actualitzar a la base de dades el pressupost pasat.
        /// </summary>
        /// <param name="pre"></param>
        public void ActualitzaPressupost(ServeiMudances.Pressupost pre)
        {
            Sync.SyncTables st = new SyncTables();
            st.guardaPressupost(pre, true);
        }

        /// <summary>
        /// Funció encarregada de retornar el següent número a assignar a les línies creades al terminal.
        /// </summary>
        public int NumeroLinia()
        {
            WP8_Configuration config = this.CarregaConfiguracio();
            config.ContadorLinies--;
            GuardaConfiguracio(config);
            return config.ContadorLinies;
        }

        /// <summary>
        /// Funció encarregada de retornar el següent número a assignar a les fotografies creades al terminal.
        /// </summary>
        public int NumeroFoto()
        {
            WP8_Configuration config = this.CarregaConfiguracio();
            config.ContadorFotos--;
            GuardaConfiguracio(config);
            return config.ContadorFotos;
        }
    }
}
