﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WP_DAO.Sync
{
    /// <summary>
    /// Classe encarregada de la sincronització de les dades entre el servidor de l'aplicació i la base de dades local.
    /// </summary>
    class SyncTables
    {
        ServeiMudances.ServeiMudancesClient cli;

        /// <summary>
        /// Constructor de la classe.
        /// </summary>
        public SyncTables()
        { }

        /// <summary>
        /// Constructor de la classe que rep com a paràmetre el client del servei WCF.
        /// </summary>
        public SyncTables(ServeiMudances.ServeiMudancesClient _cli)
        {
            cli = _cli;
            cli.getUsersCompleted += cli_getUsersCompleted;
            cli.getVehiclesCompleted += cli_getVehiclesCompleted;
            cli.getOperarisCompleted += cli_getOperarisCompleted;
            cli.getClientsCompleted += cli_getClientsCompleted;

            cli.getPressupostosCompleted += cli_getPressupostosCompleted;
        }

        /// <summary>
        /// Funció encarregada de la sincronització d'entrada de tota la informació del servidor.
        /// </summary>
        public void GetAllTables()
        {
            this.GetUsuaris();
            this.GetVehicles();
            this.GetOperaris();
            this.GetClients();

            this.GetPressupostos();
        }

        /// <summary>
        /// Funció encarregada de la sincronització de la taula d'usuaris.
        /// </summary>
        public void GetUsuaris()
        {
            cli.getUsersAsync();
        }
        /// <summary>
        /// Funció que s'executa quan el servei ha retornat la llista d'usuaris.
        /// S'encarrega d'actualitzar els objectes a la base de dades local.
        /// </summary>
        void cli_getUsersCompleted(object sender, ServeiMudances.getUsersCompletedEventArgs e)
        {
            if (!e.Cancelled)
            {
                BaseDades.Main bdMain = new BaseDades.Main();
                foreach (var user in e.Result)
                {
                    bdMain.InsertObject(ConversorsServeiBD.ServeiToWP8(user));
                }
            }
        }

        /// <summary>
        /// Funció encarregada de la sincronització de la taula de vehicles.
        /// </summary>
        public void GetVehicles()
        {
            cli.getVehiclesAsync();
        }
        /// <summary>
        /// Funció que s'executa quan el servei ha retornat la llista de vehicles.
        /// S'encarrega d'actualitzar els objectes a la base de dades local.
        /// </summary>
        void cli_getVehiclesCompleted(object sender, ServeiMudances.getVehiclesCompletedEventArgs e)
        {
            if (!e.Cancelled)
            {
                BaseDades.Main bdMain = new BaseDades.Main();
                foreach (var veh in e.Result)
                {
                    bdMain.InsertObject(ConversorsServeiBD.ServeiToWP8(veh));
                }
            }
        }

        /// <summary>
        /// Funció encarregada de la sincronització de la taula de operaris.
        /// </summary>
        public void GetOperaris()
        {
            cli.getOperarisAsync();
        }
        /// <summary>
        /// Funció que s'executa quan el servei ha retornat la llista d'operaris.
        /// S'encarrega d'actualitzar els objectes a la base de dades local.
        /// </summary>
        void cli_getOperarisCompleted(object sender, ServeiMudances.getOperarisCompletedEventArgs e)
        {
            if (!e.Cancelled)
            {
                BaseDades.Main bdMain = new BaseDades.Main();
                foreach (var op in e.Result)
                {
                    bdMain.InsertObject(ConversorsServeiBD.ServeiToWP8(op));
                }
            }
        }

        /// <summary>
        /// Funció encarregada de la sincronització de la taula de clients.
        /// </summary>
        public void GetClients()
        {
            cli.getClientsAsync();
        }
        /// <summary>
        /// Funció que s'executa quan el servei ha retornat la llista de clients.
        /// S'encarrega d'actualitzar els objectes a la base de dades local.
        /// </summary>
        void cli_getClientsCompleted(object sender, ServeiMudances.getClientsCompletedEventArgs e)
        {
            if (!e.Cancelled)
            {
                BaseDades.Main bdMain = new BaseDades.Main();
                foreach (var user in e.Result)
                {
                    bdMain.InsertObject(ConversorsServeiBD.ServeiToWP8(user));
                }
            }
        }

        /// <summary>
        /// Funció encarregada de la sincronització de la taula de pressupostos.
        /// </summary>
        public void GetPressupostos()
        {
            cli.getPressupostosAsync();
        }
        /// <summary>
        /// Funció que s'executa quan el servei ha retornat la llista de pressupostos.
        /// </summary>
        void cli_getPressupostosCompleted(object sender, ServeiMudances.getPressupostosCompletedEventArgs e)
        {
            if (!e.Cancelled)
            {
                foreach (var pre in e.Result)
                {
                    this.guardaPressupost(pre, false);
                }
            }
        }
        /// <summary>
        /// Funció encarregada d'actualitzar els pressupostos a la base de dades local.
        /// </summary>
        /// <param name="pre"></param>
        /// <param name="actualitzat"></param>
        internal void guardaPressupost(ServeiMudances.Pressupost pre, bool actualitzat)
        {
            BaseDades.Main bdMain = new BaseDades.Main();
            bdMain.DeletePressupost(pre.Id);

            bdMain.InsertObject(ConversorsServeiBD.ServeiToWP8(pre, actualitzat));

            foreach (var op in pre.operaris)
            {
                BaseDades.WP8_OperarisPressupost opPre = new BaseDades.WP8_OperarisPressupost();
                opPre.PressupostId = pre.Id;
                opPre.OperariId = op.Id;
                bdMain.InsertObject(opPre);
            }
            foreach (var veh in pre.vehicles)
            {
                BaseDades.WP8_VehiclesPressupost vehPre = new BaseDades.WP8_VehiclesPressupost();
                vehPre.PressupostId = pre.Id;
                vehPre.VehicleId = veh.Id;
                bdMain.InsertObject(vehPre);
            }
            foreach (var lin in pre.linies)
            {
                bdMain.InsertObject(ConversorsServeiBD.ServeiToWP8(pre, lin));
                foreach (var img in lin.imatges)
                {
                    bdMain.InsertObject(ConversorsServeiBD.ServeiToWP8(lin, img));
                }
            }
        }

        /// <summary>
        /// Funció encarregada d'actualitzar els pressupostos modificats a la base de dades general de l'aplicació.
        /// </summary>
        public void ActualitzaPressupostos()
        {
            WP_DAO.BaseDades.Main bd = new BaseDades.Main();
            foreach (WP_DAO.BaseDades.WP8_Pressupost trasllat in bd.GetPressupostosActualitzats())
            {
                cli.updatePressupostAsync(ConversorsBDServei.WP8ToServei(trasllat));
            }
        }
    }
}
